package com.appiancorp.analytics.excel_export;

import java.sql.Connection;
import java.sql.SQLException;

import javax.naming.Context;

import org.apache.log4j.Logger;

import com.appiancorp.suiteapi.common.Name;
import com.appiancorp.suiteapi.content.ContentConstants;
import com.appiancorp.suiteapi.content.ContentService;
import com.appiancorp.suiteapi.knowledge.Document;
import com.appiancorp.suiteapi.knowledge.DocumentDataType;
import com.appiancorp.suiteapi.knowledge.FolderDataType;
import com.appiancorp.suiteapi.process.exceptions.SmartServiceException;
import com.appiancorp.suiteapi.process.framework.AppianSmartService;
import com.appiancorp.suiteapi.process.framework.Input;
import com.appiancorp.suiteapi.process.framework.MessageContainer;
import com.appiancorp.suiteapi.process.framework.Required;
import com.appiancorp.suiteapi.process.palette.PaletteInfo;

@PaletteInfo(paletteCategory = "Appian Smart Services", palette = "Document Generation")
public class ExportSqlToFlatFile_v2 extends AppianSmartService{

	private static final Logger LOG = Logger.getLogger(ExportSqlToFlatFile_v2.class);
	private String jndiName;
	private String sql;
	private String document_name_to_create;
	private Long document_save_directory;
	private Boolean include_header_row;
	private String delimiter;
	private String extension;
	private Long document_to_overwrite;
	private Long output_document;
	private String end_of_line;

	private Context ctx;

	private ContentService cs;

	public ExportSqlToFlatFile_v2(ContentService cs, Context ctx) {
		super();
		this.cs = cs;
		this.ctx = ctx;
	}


	@Override
	public void run() throws SmartServiceException {
		if (document_to_overwrite == null || document_to_overwrite.intValue() == 0) {
			if (document_save_directory == null) {
				String msg = "Attempt to export to a CSV with an invalid folder or document id.  Either the 'Folder' or 'Document_To_Override' parameters are required. ";
				throw new SmartServiceException.Builder(getClass(),
						new Exception(msg)).build();
			} else if (document_name_to_create == null || "".equals(document_name_to_create)) {
				String msg = "Attempt to export to a CSV with an invalid document name.  When the 'Folder' parameter is specified, the 'Document_Name' parameter is required. ";
				throw new SmartServiceException.Builder(getClass(),
						new Exception(msg)).build();
			} 
		} else if (document_to_overwrite != null){
			try{
				if(extension != ((Document)cs.getVersion(document_to_overwrite, ContentConstants.VERSION_CURRENT)).getExtension());
				String msg = "Attempt to export to an existing file with a different extension. ";
				throw new SmartServiceException.Builder(getClass(),
						new Exception(msg)).build();
			}catch (Exception e){
				LOG.error(e,e);
			}

		}

		StringBuffer contents = null;
		Connection conn = null;
		try {
			conn = DBUtils.getConnection(jndiName, ctx);
		} catch (Exception e) {
			LOG.error(e, e);
			throw new SmartServiceException.Builder(getClass(), e).build();
		}
		try {
			contents = DBUtils.exportData(conn, sql, delimiter, include_header_row, end_of_line);
		} catch (Exception e) {
			LOG.error(e, e);
			contents = null;
		} finally {
			try {
				conn.close();
			} catch (SQLException e) {
				LOG.error(e,e);
			}
		}

		if (contents != null) {
			try {
				output_document = DocUtils.registerDocument(cs, document_to_overwrite, document_save_directory, document_name_to_create, extension, contents.toString());
			} catch (Exception e) {
				LOG.error(e,e);
			}
		}

	}
	public void onSave(MessageContainer messages) {
	}

	public void validate(MessageContainer messages) {
	}

	@Input(required = Required.ALWAYS)
	@Name("jndiName")
	public void setJndiName(String val) {
		this.jndiName = val;
	}

	@Input(required = Required.ALWAYS)
	@Name("sql")
	public void setSql(String val) {
		this.sql = val;
	}

	@Input(required = Required.OPTIONAL)
	@Name("document_name_to_create")
	public void setDocument_name_to_create(String val) {
		this.document_name_to_create = val;
	}

	@Input(required = Required.OPTIONAL)
	@Name("document_save_directory")
	@FolderDataType
	public void setDocument_save_directory(Long val) {
		this.document_save_directory = val;
	}

	@Input(required = Required.ALWAYS)
	@Name("delimiter")
	public void setDelimiter(String val) {
		this.delimiter = val;
	}

	@Input(required = Required.ALWAYS)
	@Name("extension")
	public void setExtension(String val) {
		this.extension = val;
	}
	
	@Input(required = Required.ALWAYS)
	@Name("end_of_line")
	public void setEndOfLine(String val) {
		this.end_of_line = val;
	}

	@Input(required = Required.OPTIONAL)
	@Name("include_header_row")
	public void setInclude_header_row(Boolean val) {
		this.include_header_row = val;
	}

	@Input(required = Required.OPTIONAL)
	@Name("document_to_overwrite")
	@DocumentDataType
	public void setDocument_to_overwrite(Long val) {
		this.document_to_overwrite = val;
	}

	@Name("output_document")
	@DocumentDataType
	public Long getOutput_document() {
		return output_document;
	}
}