package com.appiancorp.servlet.custom;

import static org.junit.Assert.*;

import java.text.DateFormat;
import java.text.ParseException;
import java.util.Locale;
import java.util.TimeZone;

import org.junit.Before;
import org.junit.Test;

import com.appiancorp.process.expression.ExpressionEvaluationException;
import com.appiancorp.suiteapi.cfg.ConfigurationLoader;
import com.appiancorp.suiteapi.common.exceptions.InvalidGroupException;
import com.appiancorp.suiteapi.common.exceptions.InvalidVersionException;
import com.appiancorp.suiteapi.common.exceptions.PrivilegeException;
import com.appiancorp.suiteapi.content.exceptions.InvalidContentException;
import com.appiancorp.services.ServiceContext;
import com.appiancorp.services.WebServiceContextFactory;

public class ExcellFormatterTestClass {

	/*
	 * Some very simple test cases for the Excel Formatter class.
	 * 
	 * May need to add Appian classpath.jar to JUnit under Run As -> Run Configurations -> Classpath -> User Entries -> Add External JARs
	 * 
	 * Note that the username and expected inputs and outputs for some tests will need to be updated to match the development environment.
	 * 
	 */
	
	private String username;
	private ServiceContext sc;
	
	@Before
	public void setUp() throws Exception {
		
		username = "";		//Set username here
		if (username.equals("")) {
			System.out.println("Update the username in ExcelFormatterTestClass.setUp()");
			System.exit(0);
		}
		
		ConfigurationLoader.initializeConfigurations();
		sc = WebServiceContextFactory.getServiceContext(username);
	
	}

	@Test
	public void testRetrieveSingleGroupName() throws PrivilegeException, InvalidGroupException {		
		Long[] groupIds = { new Long(6) };	
		assertEquals("Designers", ExcelFormatter.retrieveGroupNames(groupIds, sc));
	}
	
	@Test
	public void testRetrieveGroupNames() throws PrivilegeException, InvalidGroupException {		
		Long[] groupIds = { new Long(6), new Long(1) };	
		assertEquals("Designers;Portal Administrators", ExcelFormatter.retrieveGroupNames(groupIds, sc));
	}

	@Test
	public void testRetrieveSingleUserNameFormatted() {
		String[] uname = { "tom.ryan" };					//replace with a username that exists in the test environment
		String formattedName = "Tom Ryan"; 					//replace with the corresponding Firstname Lastname
		assertEquals(formattedName, ExcelFormatter.retrieveUserNamesFormatted(uname, sc));
	}
	
	@Test
	public void testRetrieveUserNamesFormatted() {
		String[] unames = { "tom.ryan", "test.user" };				//replace with usernames that exist in the test environment
		String formattedNames = "Tom Ryan;Test User"; 				//replace with the corresponding Firstname Lastname
		assertEquals(formattedNames, ExcelFormatter.retrieveUserNamesFormatted(unames, sc));
	}

	@Test
	public void testFormatSingleDate() {
		String[] date = { "2015-08-10" };
		try {
			assertEquals("Aug 10, 2015", ExcelFormatter.formatDates(date, DateFormat.MEDIUM, Locale.ENGLISH));
		} catch (ParseException e) {
			fail("Parse Exception: " + e);
		}
	}
	
	@Test
	public void testFormatDates() {
		String[] dates = { "1999-05-30", "2015-08-10" };
		try {
			assertEquals("May 30, 1999;Aug 10, 2015", ExcelFormatter.formatDates(dates, DateFormat.MEDIUM, Locale.ENGLISH));
		} catch (ParseException e) {
			fail("Parse Exception: " + e);
		}
	}

	@Test
	public void testFormatSingleDateTime() {
		String[] dateTime = { "2015-08-10 06:36:01" };
		TimeZone tz = TimeZone.getTimeZone("Australia/Sydney");
		try {
			assertEquals("Aug 10, 2015 6:36:01 AM", 
					ExcelFormatter.formatDateTimes(dateTime, DateFormat.MEDIUM, DateFormat.MEDIUM, Locale.ENGLISH, tz));
		} catch (ParseException e) {
			fail("Parse Exception: " + e);
		}
	}
	
	@Test
	public void testFormatDateTimes() {
		String[] dateTimes = { "2015-08-10 06:36:01", "2015-08-10 16:36:01" };
		TimeZone tz = TimeZone.getTimeZone("Australia/Sydney");
		try {
			assertEquals("Aug 10, 2015 6:36:01 AM;Aug 10, 2015 4:36:01 PM", 
					ExcelFormatter.formatDateTimes(dateTimes, DateFormat.MEDIUM, DateFormat.MEDIUM, Locale.ENGLISH, tz));
		} catch (ParseException e) {
			fail("Parse Exception: " + e);
		}
	}

	@Test
	public void testFormatDuration1() {
		String d = "1";
		try {
			assertEquals("1 days 0.0 secs", ExcelFormatter.formatDuration(d, sc, Locale.ENGLISH));
		} catch (ExpressionEvaluationException e) {
			fail("Expression Evaluation Exception: " + e);
		}
	}
	
	@Test
	public void testFormatDuration2() {
		String d = "5.12356";
		try {
			assertEquals("5 days 2 hrs 57 mins 55.584 secs", ExcelFormatter.formatDuration(d, sc, Locale.ENGLISH));
		} catch (ExpressionEvaluationException e) {
			fail("Expression Evaluation Exception: " + e);
		}
	}

	@Test
	public void testFormatContentObjectDocument() {
		String docName = "template.xlsx";
		Long[] docIds = { new Long(3855) };
		try {
			assertEquals(docName, ExcelFormatter.formatContentObject(docIds, sc));
		} catch (PrivilegeException e) {
			fail("Privilege Exception: " + e);
		} catch (InvalidContentException e) {
			fail("Invalide Content Exception: " + e);
		} catch (InvalidVersionException e) {
			fail("Invalid Version Exception: " + e);
		}
	}
	
	@Test
	public void testFormatContentObjectFolder() {
		String docName = "Temporary Documents";
		Long[] docIds = { new Long(3852) };
		try {
			assertEquals(docName, ExcelFormatter.formatContentObject(docIds, sc));
		} catch (PrivilegeException e) {
			fail("Privilege Exception: " + e);
		} catch (InvalidContentException e) {
			fail("Invalide Content Exception: " + e);
		} catch (InvalidVersionException e) {
			fail("Invalid Version Exception: " + e);
		}
	}
	
	@Test
	public void testFormatContentObjectFolderAndDoc() {
		String docName = "Temporary Documents;template.xlsx";
		Long[] docIds = { new Long(3852), new Long(3855) };
		try {
			assertEquals(docName, ExcelFormatter.formatContentObject(docIds, sc));
		} catch (PrivilegeException e) {
			fail("Privilege Exception: " + e);
		} catch (InvalidContentException e) {
			fail("Invalide Content Exception: " + e);
		} catch (InvalidVersionException e) {
			fail("Invalid Version Exception: " + e);
		}
	}

	@Test
	public void testFormatSingleInteger() {
		Long[] l = { new Long(1523243) };
		assertEquals("1523243", ExcelFormatter.formatIntegers( l ));
	}
	
	@Test
	public void testFormatIntegers() {
		Long[] l = { new Long(1523243), new Long(-1) };
		assertEquals("1523243;-1", ExcelFormatter.formatIntegers( l ));
	}

	@Test
	public void testFormatSingleDecimal() {
		Double[] d = { new Double(155.004) };
		assertEquals("155.004", ExcelFormatter.formatDecimals(d, ""));
	}
	
	@Test
	public void testFormatDecimals() {
		Double[] d = { new Double(0.04), new Double(-0.1) };
		assertEquals("4.0%;-10.0%", ExcelFormatter.formatDecimals(d, "percentage"));
	}

	@Test
	public void testFormatSingleString() {
		String[] s = { "Here is a string." };
		assertEquals("Here is a string.", ExcelFormatter.formatString(s));
	}
	
	@Test
	public void testFormatStrings() {
		String[] s = { "Here is a string.", "Here is another." };
		assertEquals("Here is a string.;Here is another.", ExcelFormatter.formatString(s));
	}

	@Test
	public void testFormatBooleanFalse() {
		Long[] l = { new Long(0) };
		assertEquals("False", ExcelFormatter.formatBoolean(l));
	}
	
	@Test
	public void testFormatBooleanTrue() {
		Long[] l = { new Long(1) };
		assertEquals("True", ExcelFormatter.formatBoolean(l));
	}
	
	@Test
	public void testFormatBooleans() {
		Long[] l = { new Long(0), new Long(1) };
		assertEquals("False;True", ExcelFormatter.formatBoolean(l));
	}

}
