package com.appiancorp.analytics.excel_export;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.DataFormat;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.usermodel.WorkbookFactory;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import com.appiancorp.suiteapi.common.LocalObject;
import com.appiancorp.suiteapi.common.Name;
import com.appiancorp.suiteapi.common.ObjectTypeMapping;
import com.appiancorp.suiteapi.content.Content;
import com.appiancorp.suiteapi.content.ContentConstants;
import com.appiancorp.suiteapi.content.ContentFilter;
import com.appiancorp.suiteapi.content.ContentService;
import com.appiancorp.suiteapi.knowledge.Document;
import com.appiancorp.suiteapi.knowledge.DocumentDataType;
import com.appiancorp.suiteapi.knowledge.FolderDataType;
import com.appiancorp.suiteapi.process.GenericTypedVariable;
import com.appiancorp.suiteapi.process.analytics2.Column;
import com.appiancorp.suiteapi.process.analytics2.Filter;
import com.appiancorp.suiteapi.process.analytics2.ProcessAnalyticsService;
import com.appiancorp.suiteapi.process.analytics2.ProcessReport;
import com.appiancorp.suiteapi.process.analytics2.ReportData;
import com.appiancorp.suiteapi.process.analytics2.ReportResultPage;
import com.appiancorp.suiteapi.process.analytics2.SimpleColumnFilter;
import com.appiancorp.suiteapi.process.exceptions.SmartServiceException;
import com.appiancorp.suiteapi.process.framework.AppianSmartService;
import com.appiancorp.suiteapi.process.framework.Input;
import com.appiancorp.suiteapi.process.framework.MessageContainer;
import com.appiancorp.suiteapi.process.framework.Required;
import com.appiancorp.suiteapi.process.framework.SmartServiceContext;
import com.appiancorp.suiteapi.process.palette.PaletteInfo;
import com.appiancorp.suiteapi.type.TypedValue;
//import com.appiancorp.common.SystemMessageResolver;

/**
 *
 * @author sathya.srinivasan
 * Smart service to export Appian report to Excel
 */
@PaletteInfo(paletteCategory = "Appian Smart Services", palette = "Document Generation")
public class ExportReportDataToExcel extends AppianSmartService {

  private static final Logger LOG = Logger
      .getLogger(ExportReportDataToExcel.class);
  private final SmartServiceContext smartServiceCtx;
  private Long excel_template;
  private String[] cell_keys;
  private String[] cell_values;
  private Long document_to_override;
  private Long folder;
  private String new_document_name;
  private String report_starting_cel;
  private Long report;
  private Long document;
  private String[] filterColumns;
  private TypedValue[] filterValues;
  private TypedValue reportContext;
  private boolean includeHeaderRow = true;
  private Long sheetNumber =0L;

  private ProcessAnalyticsService pas;
  private ContentService cs;

  public ExportReportDataToExcel(ContentService cs,
      ProcessAnalyticsService pas,
      SmartServiceContext smartServiceCtx) {
    super();
    this.smartServiceCtx = smartServiceCtx;
    this.cs = cs;
    this.pas = pas;
  }

  @Override
  public void run() throws SmartServiceException {

    Locale currentLocale = smartServiceCtx.getUserLocale() != null ? smartServiceCtx
        .getUserLocale() : smartServiceCtx.getPrimaryLocale();

    try {
      Workbook wb = new XSSFWorkbook();
      wb.createSheet("Sheet1");
      if (excel_template != null) {
      	Document xls = (Document) cs.getVersion(excel_template, ContentConstants.VERSION_CURRENT);
       	String documentPath = cs.getInternalFilename(xls.getCurrentContentId());
        FileInputStream fis = new FileInputStream(documentPath);
        wb = WorkbookFactory.create(fis);
        fis.close();
      }

      Sheet s = wb.getSheetAt(this.sheetNumber.intValue());


      if (cell_keys != null && cell_values != null
          && cell_keys.length == cell_values.length) {
        for (int i = 0; i < cell_keys.length; i++) {
          Cell a = ExportHelper.getCell(s, cell_keys[i]);
          a.setCellValue(cell_values[i]);
        }
      }

      String dateformat = "d/m/yyyy";
      if (currentLocale.equals(Locale.US)) {
        dateformat = "m/d/yyyy";
      }
      DataFormat df = wb.createDataFormat();
      CellStyle cellStyleDateAndTime = wb.createCellStyle();
      cellStyleDateAndTime.setDataFormat(df.getFormat(dateformat + " h:mm"));
      CellStyle cellStyleDate = wb.createCellStyle();
      cellStyleDate.setDataFormat(df.getFormat(dateformat));



      if (report!= null && report.intValue()>0) {
        ProcessReport pr = loadReportFromArf(report);
        ProcessReport pr2 = pr.copy();

        ReportData reportData = pr2.getData();

        if (reportContext != null && reportContext.getValue() != null) {
          LocalObject[] lo = new LocalObject[1];

          lo[0] = new LocalObject();

          switch (reportData.getContextType()) {
          case ReportData.CONTEXT_TYPE_SYSTEM:
            break;
          case ReportData.CONTEXT_TYPE_PROCESS_MODEL:
          case ReportData.CONTEXT_TYPE_PROCESS_MODEL_VERSION:
            lo[0].setType(ObjectTypeMapping.TYPE_BPM_PROCESS_MODEL);
            lo[0].setId(Long.parseLong("" + reportContext.getValue()));
            break;
          case ReportData.CONTEXT_TYPE_NODE:
            break;
          case ReportData.CONTEXT_TYPE_PROCESS:
            lo[0].setType(ObjectTypeMapping.TYPE_BPM_PROCESS);
            lo[0].setId(Long.parseLong("" + reportContext.getValue()));
            break;
          case ReportData.CONTEXT_TYPE_CREATED_BY:
          case ReportData.CONTEXT_TYPE_LAST_MOD_BY:
          case ReportData.CONTEXT_TYPE_INITIATED_BY:
          case ReportData.CONTEXT_TYPE_ATTRIBUTED_TO_USER:
          case ReportData.CONTEXT_TYPE_TASK_OWNER:
            lo[0].setType(ObjectTypeMapping.TYPE_USER);
            lo[0].setStringId("" + reportContext.getValue());
            break;
          case ReportData.CONTEXT_TYPE_ASSIGNED_GROUP:
          case ReportData.CONTEXT_TYPE_GROUP_MEMBERS:
            lo[0].setType(ObjectTypeMapping.TYPE_GROUP);
            lo[0].setId(Long.parseLong("" + reportContext.getValue()));
            break;
          }
          reportData.setContext(lo);
        }

        if (filterColumns != null && filterColumns.length > 0) {
          if (filterValues == null || filterValues.length != filterColumns.length) {
            LOG.error("filter columns array and filter values arrays must be of the same length.");
          } else {
            Filter[] filters = reportData.getBaseFilters();
            ArrayList filtersList = new ArrayList();
            filtersList.addAll(Arrays.asList(filters));
            for (int i = 0; i < filterColumns.length; i++) {

              SimpleColumnFilter filter = new SimpleColumnFilter();
              filter.setApply(true);
              Integer columnLocalId = null;
              for (Column c : reportData.getColumns()) {
                String name = c.getName();
                if (name.toLowerCase().equals(filterColumns[i].toLowerCase())) {
                  columnLocalId = c.getLocalId();
                  break;
                }
              }
              if (columnLocalId == null) {
                LOG.error("cannot find column for name " + filterColumns[i]);
              } else {
                filter.setColumnLocalId(columnLocalId);
                filter.setComparison(SimpleColumnFilter.ComparisonType.EQUAL
                    .getIntValue());
                GenericTypedVariable gtv = new GenericTypedVariable();
                gtv.setType(filterValues[i].getInstanceType());
                gtv.setValue(filterValues[i].getValue());
                filter.setConstant(gtv);
                filtersList.add(filter);
              }

            }
            reportData.setBaseFilters((Filter[]) filtersList
                .toArray(new Filter[filtersList.size()]));

          }
        }

        Long batchSize = 1000l;
        reportData.setBatchSize(batchSize.intValue());
        reportData.setStartIndex(0);

        LOG.debug("Report data to execute : " + reportData);

        ReportResultPage reportResultPage = pas
            .getReportPageWithTypedValues(reportData);

        Column[] columns = reportData.getColumns();

        report_starting_cel = StringUtils.isEmpty(report_starting_cel)?"a1":report_starting_cel;
        Cell reportStart = ExportHelper.getCell(s, report_starting_cel);
        Cell current = reportStart;
        if (includeHeaderRow) {
          for (Column c : columns) {
            if (c.getShow()) {
              current.setCellValue(c.getName());
              Cell next = current.getRow().getCell(current.getColumnIndex() + 1);
              if (next == null) {
                next = current.getRow().createCell(current.getColumnIndex() + 1);
              }
              current = next;
              // s.getRow(0).get
            }
          }
          current = ExportHelper.getCell(s, reportStart.getRowIndex() + 1,
              reportStart.getColumnIndex());
        }

        int rowi = current.getRowIndex();
        int celli_s = reportStart.getColumnIndex();
        for (int i = 0; i < reportResultPage.getResults().length; i++) {
          HashMap rowData = (HashMap) reportResultPage.getResults()[i];
          int celli = celli_s;
          for (Column c : columns) {
            if (c.getShow()) {
              current = ExportHelper.getCell(s, rowi, celli);
              TypedValue tv = (TypedValue) rowData.get(c.getStringId());
              if (tv.getValue() instanceof String) {
                current.setCellValue((String) tv.getValue());
              } else if (tv.getValue() instanceof Timestamp) {
                current.setCellValue((Timestamp) tv.getValue());
                current.setCellStyle(cellStyleDateAndTime);
              } else if (tv.getValue() instanceof Date) {
                current.setCellValue((Date) tv.getValue());
                current.setCellStyle(cellStyleDate);
              } else if (tv.getValue() instanceof Number) {
                current
                    .setCellValue(Double.parseDouble(tv.getValue().toString()));
              } else if (tv.getValue() instanceof String[]) {
                String[] a = (String[]) tv.getValue();
                if (a.length == 1) {
                  current.setCellValue(a[0]);
                } else {
                  current.setCellValue(StringUtils.join(a, ", "));
                }
              }

              else if (tv.getValue() instanceof Long[]) {
                  Long[] a = (Long[]) tv.getValue();
                  if (a.length == 1) {
                    current.setCellValue("" + a[0]);
                  } else {
                    current.setCellValue(StringUtils.join(a, ", "));
                  }
              } else if (tv.getValue() instanceof LocalObject[]) {
                LocalObject[] a = (LocalObject[]) tv.getValue();
                if (a.length == 1) {
                  if (a[0].getId() == null) {
                    current.setCellValue(a[0].getStringId());
                  } else {
                    current.setCellValue(a[0].getId());
                  }
                } else {
                  current.setCellValue(StringUtils.join(a, ", "));
                }
              } else if (tv.getValue() == null) {
                current.setCellValue("");
              } else {
                LOG.error("cannot print this type: "
                    + tv.getValue().getClass().getName());
              }
              Cell next = current.getRow().getCell(current.getColumnIndex() + 1);
              if (next == null) {
                next = current.getRow().createCell(current.getColumnIndex() + 1);
              }
              current = next;
              celli++;
              // s.getRow(0).get
            }
          }
          rowi++;

        }
      }

      Document d = registerDocument();
      File file = new File(d.getInternalFilename());
      FileOutputStream fos = new FileOutputStream(file);
      wb.write(fos);
      fos.close();

      d.setSize((int) file.length());
      cs.updateFields(d, new Integer[] { Document.COLUMN_SIZE },
          ContentConstants.UNIQUE_NONE);
      document = d.getId();
    } catch (Exception e) {
      LOG.error(e, e);
      throw createException(e, "error.export.general", e.getMessage());
    }

  }

  private SmartServiceException createException(Throwable t, String key,
      Object... args) {
    return new SmartServiceException.Builder(getClass(), t).userMessage(key,
        args).build();
  }



  private Document registerDocument() throws Exception {

    String name = new_document_name;
    String extension = "xlsx";

    ContentFilter cf = new ContentFilter(ContentConstants.TYPE_DOCUMENT);
    cf.setName(name);
    cf.setExtension(new String[] { extension });

    Long docId = null;
    if (document_to_override != null) {
      Document d = (Document) cs.getVersion(document_to_override,
          ContentConstants.VERSION_CURRENT);
      cs.createVersion(d, ContentConstants.UNIQUE_FOR_PARENT);
      docId = d.getId();
    } else {
      Content[] children = cs.getChildren(folder, cf,
          ContentConstants.GC_MOD_NORMAL);
      if (children == null || children.length == 0) {
        Document d = new Document();
        d.setName(name);
        d.setExtension(extension);
        d.setSize(1);
        d.setParent(folder);
        d.setState(ContentConstants.STATE_ACTIVE_PUBLISHED);
        docId = cs.create(d, ContentConstants.UNIQUE_FOR_PARENT);
      } else {
        Document d = (Document) children[0];
        cs.createVersion(d, ContentConstants.UNIQUE_FOR_PARENT);
        docId = d.getId();
      }
    }

    if (docId != null) {
      Document d = cs.download(docId, ContentConstants.VERSION_CURRENT, false)[0];
      return d;
    }

    return null;
  }

  private ProcessReport loadReportFromArf(Long reportId) throws Exception {

    return pas.getProcessReport(reportId);

//    Document xls = cs
//        .download(reportId, ContentConstants.VERSION_CURRENT, true)[0];
//    String documentPath = xls.getInternalFilename();
//    FileInputStream fis = new FileInputStream(documentPath);
//
//    StringWriter stringWriter = new StringWriter();
//    IOUtils.copy(fis, stringWriter);
//
//    return AppianReportFileSerializer.deserialize(stringWriter.toString());

    // DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
    // DocumentBuilder db = dbf.newDocumentBuilder();
    // org.w3c.dom.Document doc = db.parse(fis);
    // doc.getDocumentElement().normalize();
    //
    // ProcessReport pr = new ProcessReport();
    //
    // ReportData data = pr.getData();
    //
    // XMLUtil.populateObject(data, doc);
    // ReportDisplay display = pr.getDisplay();
    // XMLUtil.populateObject(display, doc);
    // return pr;
  }

  @Override
  public void onSave(MessageContainer messages) {
  }

  @Override
  public void validate(MessageContainer messages) {
  }

  @Input(required = Required.OPTIONAL)
  @Name("Filter_Values")
  public void setFilterValues(TypedValue[] val) {
    this.filterValues = val;
  }

  @Input(required = Required.OPTIONAL)
  @Name("Filter_Columns")
  public void setFilterColumns(String[] val) {
    this.filterColumns = val;
  }

  @Input(required = Required.OPTIONAL)
  @Name("Excel_Base_Template")
  @DocumentDataType
  public void setExcel_template(Long val) {
    this.excel_template = val;
  }

  @Input(required = Required.OPTIONAL)
  @Name("Cell_Keys")
  public void setCell_keys(String[] val) {
    this.cell_keys = val;
  }

  @Input(required = Required.OPTIONAL)
  @Name("Cell_Values")
  public void setCell_values(String[] val) {
    this.cell_values = val;
  }

  @Input(required = Required.OPTIONAL)
  @Name("Document_to_Overwrite")
  @DocumentDataType
  public void setDocument_to_override(Long val) {
    this.document_to_override = val;
  }

  @Input(required = Required.OPTIONAL)
  @Name("New_Document_Folder")
  @FolderDataType
  public void setFolder(Long val) {
    this.folder = val;
  }

  @Input(required = Required.OPTIONAL)
  @Name("New_Document_Name")
  public void setNew_document_name(String val) {
    this.new_document_name = val;
  }

  @Input(required = Required.OPTIONAL)
  @Name("Report_Starting_Cell")
  public void setReport_starting_cel(String val) {
    this.report_starting_cel = val;
  }

  @Input(required = Required.OPTIONAL)
  @Name("Report_Context")
  public void setReportContext(TypedValue val) {
    this.reportContext = val;
  }

  @Input(required = Required.OPTIONAL)
  @Name("Report")
  @DocumentDataType
  public void setReport(Long val) {
    this.report = val;
  }

  @Name("Output_Document")
  @DocumentDataType
  public Long getDocument() {
    return document;
  }

  @Input(required = Required.ALWAYS)
  @Name("Include_Header_Row")
  public void setIncludeHeaderRow(Boolean ihr) {
    includeHeaderRow = ihr == null ? true : ihr;
  }

	@Input(required = Required.ALWAYS)
	@Name("sheetNumber")
	public void setInteger(Long val) {
		this.sheetNumber = val;
	}


}
